@@ -3,7 +3,7 @@
  * Kronolith_Driver:: defines an API for implementing storage backends
  * for Kronolith.
  *
- * $Horde: kronolith/lib/Driver.php,v 1.116.2.54 2006/10/29 14:13:20 karsten Exp $
+ * $Horde: kronolith/lib/Driver.php,v 1.116.2.59 2007/03/15 18:59:55 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Kronolith 0.1
@@ -407,12 +407,8 @@ class Kronolith_Event {
      */
     function &toiCalendar(&$calendar)
     {
-        global $prefs;
-
-        // Get a reference to the calendar driver.
-        $kronolith = &$this->getDriver();
-
         $vEvent = &Horde_iCalendar::newComponent('vevent', $calendar);
+        $v1 = $calendar->getAttribute('VERSION') == '1.0';
 
         if ($this->isAllDay()) {
             $vEvent->setAttribute('DTSTART', $this->start, array('VALUE' => 'DATE'));
@@ -424,20 +420,41 @@ class Kronolith_Event {
 
         $vEvent->setAttribute('DTSTAMP', time());
         $vEvent->setAttribute('UID', $this->_uid);
-        $vEvent->setAttribute('SUMMARY', $this->title);
+        $vEvent->setAttribute('SUMMARY', $v1 ? $this->title : String::convertCharset($this->title, NLS::getCharset(), 'utf-8'));
         $vEvent->setAttribute('TRANSP', 'OPAQUE');
+        $name = Kronolith::getUserName($this->getCreatorId());
+        if (!$v1) {
+            $name = String::convertCharset($name, NLS::getCharset(), 'utf-8');
+        }
         $vEvent->setAttribute('ORGANIZER',
-                              'MAILTO:' . Kronolith::getUserEmail($this->getCreatorId()),
-                              array('CN' => Kronolith::getUserName($this->getCreatorId())));
+                              'mailto:' . Kronolith::getUserEmail($this->getCreatorId()),
+                              array('CN' => $name));
         if (!empty($this->description)) {
-            $vEvent->setAttribute('DESCRIPTION', $this->description);
+            $vEvent->setAttribute('DESCRIPTION', $v1 ? $this->description : String::convertCharset($this->description, NLS::getCharset(), 'utf-8'));
         }
         $categories = $this->getCategory();
         if (!empty($categories)) {
-            $vEvent->setAttribute('CATEGORIES', $categories);
+            $vEvent->setAttribute('CATEGORIES', $v1 ? $categories : String::convertCharset($categories, NLS::getCharset(), 'utf-8'));
         }
         if (!empty($this->location)) {
-            $vEvent->setAttribute('LOCATION', $this->location);
+            $vEvent->setAttribute('LOCATION', $v1 ? $this->location : String::convertCharset($this->location, NLS::getCharset(), 'utf-8'));
+        }
+
+        // Status.
+        switch ($this->getStatus()) {
+        case KRONOLITH_STATUS_TENTATIVE:
+            $vEvent->setAttribute('STATUS', 'TENTATIVE');
+            break;
+        case KRONOLITH_STATUS_CONFIRMED:
+            $vEvent->setAttribute('STATUS', 'CONFIRMED');
+            break;
+        case KRONOLITH_STATUS_CANCELLED:
+            if ($v1) {
+                $vEvent->setAttribute('STATUS', 'DECLINED');
+            } else {
+                $vEvent->setAttribute('STATUS', 'CANCELLED');
+            }
+            break;
         }
 
         // Attendees.
@@ -445,38 +462,70 @@ class Kronolith_Event {
             $params = array();
             switch ($status['attendance']) {
             case KRONOLITH_PART_REQUIRED:
-                $params['ROLE'] = 'REQ-PARTICIPANT';
+                if ($v1) {
+                    $params['EXPECT'] = 'REQUIRE';
+                } else {
+                    $params['ROLE'] = 'REQ-PARTICIPANT';
+                }
                 break;
 
             case KRONOLITH_PART_OPTIONAL:
-                $params['ROLE'] = 'OPT-PARTICIPANT';
+                if ($v1) {
+                    $params['EXPECT'] = 'REQUEST';
+                } else {
+                    $params['ROLE'] = 'OPT-PARTICIPANT';
+                }
                 break;
 
             case KRONOLITH_PART_NONE:
-                $params['ROLE'] = 'NON-PARTICIPANT';
+                if ($v1) {
+                    $params['EXPECT'] = 'FYI';
+                } else {
+                    $params['ROLE'] = 'NON-PARTICIPANT';
+                }
                 break;
             }
 
             switch ($status['response']) {
             case KRONOLITH_RESPONSE_NONE:
-                $params['PARTSTAT'] = 'NEEDS-ACTION';
-                $params['RSVP'] = 'TRUE';
+                if ($v1) {
+                    $params['STATUS'] = 'NEEDS ACTION';
+                    $params['RSVP'] = 'YES';
+                } else {
+                    $params['PARTSTAT'] = 'NEEDS-ACTION';
+                    $params['RSVP'] = 'TRUE';
+                }
                 break;
 
             case KRONOLITH_RESPONSE_ACCEPTED:
-                $params['PARTSTAT'] = 'ACCEPTED';
+                if ($v1) {
+                    $params['STATUS'] = 'ACCEPTED';
+                } else {
+                    $params['PARTSTAT'] = 'ACCEPTED';
+                }
                 break;
 
             case KRONOLITH_RESPONSE_DECLINED:
-                $params['PARTSTAT'] = 'DECLINED';
+                if ($v1) {
+                    $params['STATUS'] = 'DECLINED';
+                } else {
+                    $params['PARTSTAT'] = 'DECLINED';
+                }
                 break;
 
             case KRONOLITH_RESPONSE_TENTATIVE:
-                $params['PARTSTAT'] = 'TENTATIVE';
+                if ($v1) {
+                    $params['STATUS'] = 'TENTATIVE';
+                } else {
+                    $params['PARTSTAT'] = 'TENTATIVE';
+                }
                 break;
             }
 
-            $vEvent->setAttribute('ATTENDEE', 'MAILTO:' . $email, $params);
+            if (!$v1) {
+                $email = 'mailto:' . $email;
+            }
+            $vEvent->setAttribute('ATTENDEE', $email, $params);
         }
 
         // vCalendar 1.0 alarms. Has to be replaced with vAlarm components for
@@ -487,49 +536,10 @@ class Kronolith_Event {
 
         // Recurrence.
         if ($this->recurType) {
-            $rrule = '';
-            switch ($this->recurType) {
-            case KRONOLITH_RECUR_NONE:
-                break;
-
-            case KRONOLITH_RECUR_DAILY:
-                $rrule = 'FREQ=DAILY;INTERVAL='  . $this->recurInterval;
-                break;
-
-            case KRONOLITH_RECUR_WEEKLY:
-                $rrule = 'FREQ=WEEKLY;INTERVAL=' . $this->recurInterval . ';BYDAY=';
-                $vcaldays = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
-
-                for ($i = $flag = 0; $i <= 7 ; ++$i) {
-                    if ($this->recurOnDay(pow(2, $i))) {
-                        if ($flag) {
-                            $rrule .= ',';
-                        }
-                        $rrule .= $vcaldays[$i];
-                        $flag = true;
-                    }
-                }
-                break;
-
-            case KRONOLITH_RECUR_DAY_OF_MONTH:
-                $rrule = 'FREQ=MONTHLY;INTERVAL=' . $this->recurInterval;
-                break;
-
-            case KRONOLITH_RECUR_WEEK_OF_MONTH:
-                $vcaldays = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
-                $rrule = 'FREQ=MONTHLY;INTERVAL=' . $this->recurInterval . ';BYDAY=' .
-                    ($this->start->weekOfYear() - date('W', mktime(0, 0, 0, $this->start->month, 1,
-                                                                   $this->start->year)) + 1) .
-                    $vcaldays[$this->start->dayOfWeek()];
-                break;
-
-            case KRONOLITH_RECUR_YEARLY:
-                $rrule = 'FREQ=YEARLY;INTERVAL=' . $this->recurInterval;
-                break;
-            }
-
-            if (!empty($rrule) && !empty($this->recurEnd)) {
-                $rrule .= ';UNTIL=' . $calendar->_exportDate($this->recurEnd);
+            if ($v1) {
+                $rrule = $this->toRRule10($calendar);
+            } else {
+                $rrule = $this->toRRule20($calendar);
             }
             if (!empty($rrule)) {
                 $vEvent->setAttribute('RRULE', $rrule);
@@ -552,6 +562,111 @@ class Kronolith_Event {
         return $vEvent;
     }
 
+    function toRRule10($calendar)
+    {
+        switch ($this->recurType) {
+        case KRONOLITH_RECUR_NONE:
+            return '';
+
+        case KRONOLITH_RECUR_DAILY:
+            $rrule = 'D' . $this->recurInterval;
+            break;
+
+        case KRONOLITH_RECUR_WEEKLY:
+            $rrule = 'W' . $this->recurInterval;
+            $vcaldays = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
+
+            for ($i = 0; $i <= 7 ; ++$i) {
+                if ($this->recurOnDay(pow(2, $i))) {
+                    $rrule .= ' ' . $vcaldays[$i];
+                }
+            }
+            break;
+
+        case KRONOLITH_RECUR_DAY_OF_MONTH:
+            $rrule = 'MD' . $this->recurInterval . ' ' . trim($this->start->mday);
+            break;
+
+        case KRONOLITH_RECUR_WEEK_OF_MONTH:
+            $next = new Horde_Date($this->start);
+            $next->mday += 7;
+            $next->correct();
+
+            if ($this->start->month != $next->start->month) {
+                $p = 5;
+            } else {
+                $p = (int)($this->start->mday / 7);
+                if (($this->start->mday % 7) > 0) {
+                    $p++;
+                }
+            }
+
+            $vcaldays = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
+            $rrule = 'MP' . $this->recurInterval . ' ' . $p . '+ ' . $vcaldays[$this->start->dayOfWeek()];
+            break;
+
+        case KRONOLITH_RECUR_YEARLY:
+            $rrule = 'YM' . $this->recurInterval . ' ' . trim($this->start->month);
+            break;
+        }
+
+        return $this->hasRecurEnd() ?
+            $rrule . ' ' . $calendar->_exportDate($this->recurEnd) :
+            $rrule . ' #0';
+    }
+
+    function toRRule20($calendar)
+    {
+        switch ($this->recurType) {
+        case KRONOLITH_RECUR_NONE:
+            return '';
+
+        case KRONOLITH_RECUR_DAILY:
+            $rrule = 'FREQ=DAILY;INTERVAL='  . $this->recurInterval;
+            break;
+
+        case KRONOLITH_RECUR_WEEKLY:
+            $rrule = 'FREQ=WEEKLY;INTERVAL=' . $this->recurInterval . ';BYDAY=';
+            $vcaldays = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
+
+            for ($i = $flag = 0; $i <= 7 ; ++$i) {
+                if ($this->recurOnDay(pow(2, $i))) {
+                    if ($flag) {
+                        $rrule .= ',';
+                    }
+                    $rrule .= $vcaldays[$i];
+                    $flag = true;
+                }
+            }
+            break;
+
+        case KRONOLITH_RECUR_DAY_OF_MONTH:
+            $rrule = 'FREQ=MONTHLY;INTERVAL=' . $this->recurInterval;
+            break;
+
+        case KRONOLITH_RECUR_WEEK_OF_MONTH:
+            $vcaldays = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
+            $weekday =  date('W', mktime(0,
+                                         0,
+                                         0,
+                                         $this->start->month,
+                                         1,
+                                         $this->start->year));
+            $rrule = 'FREQ=MONTHLY;INTERVAL=' . $this->recurInterval
+                . ';BYDAY=' . ($this->start->weekOfYear() - $weekday + 1)
+                . $vcaldays[$this->start->dayOfWeek()];
+            break;
+
+        case KRONOLITH_RECUR_YEARLY:
+            $rrule = 'FREQ=YEARLY;INTERVAL=' . $this->recurInterval;
+            break;
+        }
+
+        return $this->hasRecurEnd()
+            ? $rrule . ';UNTIL=' . $calendar->_exportDate($this->recurEnd)
+            : $rrule;
+    }
+
     /**
      * Update the properties of this event from a
      * Horde_iCalendar_vevent object.
@@ -588,18 +703,6 @@ class Kronolith_Event {
         if (!is_array($desc) && !is_a($desc, 'PEAR_Error')) {
             $this->setDescription($desc);
         }
-                /* All day events are transferred by many device as
-                 * DSTART: YYYYMMDDT000000 DTEND: YYYYMMDDT2359(59|00)
-                 * Convert accordingly */
-                if (is_object($this->start)
-                    && $this->start->hour == 0  && $this->start->min == 0
-                    && $this->start->sec == 0
-                    && $this->end->hour == 23 && $this->end->sec == 59) {
-                    $this->end = new Horde_Date(array(
-                                   'year' => (int)$this->end->year,
-                                   'month' => (int)$this->end->month,
-                                   'mday' => (int)$this->end->mday + 1));
-                }
 
         // Location.
         $location = $vEvent->getAttribute('LOCATION');
@@ -625,11 +728,25 @@ class Kronolith_Event {
             if (!is_array($end)) {
                 // Date-Time field
                 $this->end = new Horde_Date($end);
+                /* All day events are transferred by many device as
+                 * DSTART: YYYYMMDDT000000 DTEND: YYYYMMDDT2359(59|00)
+                 * Convert accordingly */
+                if (is_object($this->start)
+                    && $this->start->hour == 0  && $this->start->min == 0
+                    && $this->start->sec == 0
+                    && $this->end->hour == 23 && $this->end->sec == 59) {
+                    $this->end = new Horde_Date(array(
+                                   'year' => (int)$this->end->year,
+                                   'month' => (int)$this->end->month,
+                                   'mday' => (int)$this->end->mday + 1));
+                    $this->end->correct();
+                }
             } elseif (is_array($end) && !is_a($end, 'PEAR_Error')) {
                 // Date field
                 $this->end = new Horde_Date(array('year' => (int)$end['year'],
                                                   'month' => (int)$end['month'],
                                                   'mday' => (int)$end['mday'] + 1));
+                $this->end->correct();
             }
         } else {
             $duration = $vEvent->getAttribute('DURATION');
@@ -652,8 +769,8 @@ class Kronolith_Event {
          * copy of an event can cause invitation updates to be sent from
          * people other than the original organizer. So we don't import by
          * default. However to allow updates by SyncML replication, the custom
-         * X-ATTENDEE attribute is used which has the same syntax as ATTENDEE.
-         */
+         * X-ATTENDEE attribute is used which has the same syntax as
+         * ATTENDEE. */
         $attendee = $vEvent->getAttribute('X-ATTENDEE');
         if (!is_a($attendee, 'PEAR_Error')) {
             require_once 'Horde/MIME.php';
@@ -666,7 +783,7 @@ class Kronolith_Event {
                 $params = array($params);
             }
             for ($i = 0; $i < count($attendee); ++$i) {
-                $attendee[$i] = str_replace('MAILTO:', '', $attendee[$i]);
+                $attendee[$i] = str_replace(array('MAILTO:', 'mailto:'), '', $attendee[$i]);
                 $email = MIME::bareAddress($attendee[$i]);
                 // Default according to rfc2445:
                 $attendance = KRONOLITH_PART_REQUIRED;
@@ -1087,6 +1204,11 @@ class Kronolith_Event {
         return $this->recurInterval;
     }
 
+    function hasRecurEnd()
+    {
+        return (isset($this->recurEnd) && isset($this->recurEnd->year) && $this->recurEnd->year != 9999);
+    }
+
     /**
      * Retrieve the locally unique identifier for this event.
      *
@@ -1370,11 +1492,6 @@ class Kronolith_Event {
         return in_array($keyword, $this->keywords);
     }
 
-    function hasRecurEnd()
-    {
-        return (isset($this->recurEnd) && isset($this->recurEnd->year) && $this->recurEnd->year != 9999);
-    }
-
     function isAllDay()
     {
         return ($this->start->hour == 0 && $this->start->min == 0 && $this->start->sec == 0 &&
