@@ -2,7 +2,7 @@
 /**
  * Kronolith external API interface.
  *
- * $Horde: kronolith/lib/api.php,v 1.126.2.18 2006/06/15 17:57:41 jan Exp $
+ * $Horde: kronolith/lib/api.php,v 1.126.2.25 2006/12/27 17:42:49 jan Exp $
  *
  * This file defines Kronolith's external API interface. Other applications
  * can interact with Kronolith through this API.
@@ -22,7 +22,7 @@ $_services['show'] = array(
 
 $_services['browse'] = array(
     'args' => array('path' => 'string'),
-    'type' => '{urn:hash}hashHash',
+    'type' => '{urn:horde}hashHash',
 );
 
 $_services['getFreeBusy'] = array(
@@ -37,7 +37,7 @@ $_services['listCalendars'] = array(
 
 $_services['listEvents'] = array(
     'args' => array('startstamp' => 'int', 'endstamp' => 'int', 'calendar' => 'string', 'showRecurrence' => 'string'),
-    'type' => 'array'
+    'type' => '{urn:horde}stringArray'
 );
 
 $_services['list'] = array(
@@ -430,14 +430,13 @@ function _kronolith_export($uid, $contentType)
     case 'text/x-icalendar':
     case 'text/x-vtodo':
         $share = &$kronolith_shares->getShare($event->getCalendar());
-        $identity = &$kronolith_shares->getIdentityByShare($share);
 
         require_once 'Horde/iCalendar.php';
         $iCal = new Horde_iCalendar($version);
         $iCal->setAttribute('X-WR-CALNAME', $share->get('name'));
 
         // Create a new vEvent.
-        $vEvent = &$event->toiCalendar($iCal, $identity);
+        $vEvent = &$event->toiCalendar($iCal);
         $iCal->addComponent($vEvent);
 
         return $iCal->exportvCalendar();
@@ -491,7 +490,6 @@ function _kronolith_exportCalendar($calendar, $contentType)
     case 'text/x-icalendar':
     case 'text/x-vtodo':
         $share = &$kronolith_shares->getShare($calendar);
-        $identity = &$kronolith_shares->getIdentityByShare($share);
 
         require_once 'Horde/iCalendar.php';
         $iCal = new Horde_iCalendar($version);
@@ -499,7 +497,7 @@ function _kronolith_exportCalendar($calendar, $contentType)
 
         foreach ($events as $id) {
             $event = &$kronolith->getEvent($id);
-            $vEvent = &$event->toiCalendar($iCal, $identity);
+            $vEvent = &$event->toiCalendar($iCal);
             $iCal->addComponent($vEvent);
         }
 
@@ -520,7 +518,7 @@ function _kronolith_exportCalendar($calendar, $contentType)
  */
 function _kronolith_delete($uid)
 {
-    /* Handle an arrray of UIDs for convenience of deleting multiple events at
+    /* Handle an array of UIDs for convenience of deleting multiple events at
      * once. */
     if (is_array($uid)) {
         foreach ($uid as $g) {
@@ -536,13 +534,33 @@ function _kronolith_delete($uid)
     require_once dirname(__FILE__) . '/base.php';
     global $kronolith;
 
-    $event = $kronolith->getByUID($uid);
-    if (is_a($event, 'PEAR_Error')) {
-        return $event;
+    $events = $kronolith->getByUID($uid, true);
+    if (is_a($events, 'PEAR_Error')) {
+        return $events;
     }
 
-    if (!array_key_exists($event->getCalendar(),
-                          Kronolith::listCalendars(false, PERMS_DELETE))) {
+    /* First try the user's own calendars. */
+    $ownerCalendars = Kronolith::listCalendars(true, PERMS_DELETE);
+    $event = null;
+    foreach ($events as $ev) {
+    	if (isset($ownerCalendars[$ev->getCalendar()])) {
+            $event = $ev;
+            break;
+    	}
+    }
+
+    /* If not successful, try all calendars the user has access too. */
+    if (empty($event)) {
+    	$deletableCalendars = Kronolith::listCalendars(false, PERMS_DELETE);
+    	foreach ($events as $ev) {
+            if (isset($deletableCalendars[$ev->getCalendar()])) {
+                $event = $ev;
+                break;
+            }
+    	}
+    }
+
+    if (empty($event)) {
         return PEAR::raiseError(_("Permission Denied"));
     }
 
@@ -690,26 +708,48 @@ function _kronolith_updateAttendee($response)
         return $uid;
     }
 
-    $event = $kronolith->getByUID($uid);
-    if (is_a($event, 'PEAR_Error')) {
-        return $event;
+    $events = $kronolith->getByUID($uid, true);
+    if (is_a($events, 'PEAR_Error')) {
+        return $events;
     }
 
-    if (!array_key_exists($event->getCalendar(),
-                          Kronolith::listCalendars(false, PERMS_EDIT))) {
+    /* First try the user's own calendars. */
+    $ownerCalendars = Kronolith::listCalendars(true, PERMS_EDIT);
+    $event = null;
+    foreach ($events as $ev) {
+    	if (isset($ownerCalendars[$ev->getCalendar()])) {
+            $event = $ev;
+            break;
+    	}
+    }
+
+    /* If not successful, try all calendars the user has access too. */
+    if (empty($event)) {
+    	$editableCalendars = Kronolith::listCalendars(false, PERMS_EDIT);
+    	foreach ($events as $ev) {
+            if (isset($editableCalendars[$ev->getCalendar()])) {
+                $event = $ev;
+                break;
+            }
+    	}
+    }
+
+    if (empty($event)) {
         return PEAR::raiseError(_("Permission Denied"));
     }
 
     $atnames = $response->getAttribute('ATTENDEE');
-    $atparms = $response->getAttribute('ATTENDEE', true);
-
     if (!is_array($atnames)) {
         $atnames = array($atnames);
     }
+    $atparms = $response->getAttribute('ATTENDEE', true);
+
+    $found = false;
     foreach ($atnames as $index => $attendee) {
         $attendee = str_replace('mailto:', '', String::lower($attendee));
         if ($event->hasAttendee($attendee)) {
             $event->addAttendee($attendee, KRONOLITH_PART_IGNORE, Kronolith::responseFromICal($atparms[$index]['PARTSTAT']));
+            $found = true;
         }
     }
 
@@ -718,6 +758,10 @@ function _kronolith_updateAttendee($response)
         return $result;
     }
 
+    if (!$found) {
+        return PEAR::raiseError(_("No attendees have been updated because none of the provided email addresses have been found in the event's attendees list."));
+    }
+
     return true;
 }
 
